<?php
/**
 * IMAP ACL Manipulation Plugin
 *
 * Copyright (c) 2003 Samuel Graenacher
 *
 * Licensed under the GNU GPL. For full terms see the file COPYING that came
 * with the Squirrelmail distribution.
 *
 * Developped for SM version 1.4.1
 *
 * This plugin was written and tested using cyrus ACLs. If you can test is
 * with other ACL implementations (like Communigate Pro's..) please drop me
 * a note, how well it works. Fixes and patches are even more welcome. :)
 *
 * $Id$
 */


if (!defined("SM_PATH"))
    define('SM_PATH','../../');

include_once(SM_PATH . 'functions/global.php');
include_once(SM_PATH . 'functions/i18n.php');
include_once(SM_PATH . 'include/validate.php');
include_once(SM_PATH . 'include/load_prefs.php');
include_once(SM_PATH . 'functions/page_header.php');
include_once(SM_PATH . 'functions/imap.php');

IA_init_language();

include_once(SM_PATH . 'plugins/imapacl/htmloutput.php');
include_once(SM_PATH . 'plugins/imapacl/EasyTemplateS1.inc.php');


sqsession_is_active();
sqgetGlobalVar('key', $key, SQ_COOKIE);
sqgetGlobalVar('onetimepad', $onetimepad, SQ_SESSION);
if(isset($_SESSION['part'])) {
        session_unregister('part');
}
if(isset($_SESSION['sess'])) {
        session_unregister('sess');
}

/* For later use, we need the cleartext password. */
$clearpass = OneTimePadDecrypt($key, $onetimepad);


////////////////////////////////////////////////////////////////
// Definitions
$rightsList = array(
    'l' => _("<b>List</b> - user can see the folder"),
    'r' => _("<b>Read</b> messages"),
    's' => _("Mark with <b>Seen</b>/Unseen flags"),
    'w' => _("<b>Mark</b> with other flags (e.g. Important/Answered)"),
    'i' => _("<b>Insert</b> messages"),
    'p' => _("<b>Post</b> to this folder <i>(not enforced by IMAP)</i>"),
    'c' => _("<b>Create</b> sub <b>folders</b>"),
    'd' => _("<b>Delete</b> and <b>purge</b> messages"),
    'a' => _("<b>Administer</b> - set permissions for other users")
);

$rightsListSmall = Array(
    'l' => _("Lookup"),
    'r' => _("Read"),
    's' => _("Seen"),
    'w' => _("Write"),
    'i' => _("Add"),
    'p' => _("Post"),
    'c' => _("Create"),
    'd' => _("Delete"),
    'a' => _("Admin")
);

////////////////////////////////////////////////////////////////
// Configuration
$ia_cfg_doSanity = 1;
// Set this to zero if you are certain that your users know
// what they're doing.


////////////////////////////////////////////////////////////////
// Debug Functions
//       showObject() and debugObject()
//       - debug code to show an object or array
                         
function showObject(&$obj) {
        $r = debugObject($obj);
        echo '<PRE>'.$r.'</PRE>\n';
}
                
function debugObject($obj,$indent=0) {
        if (is_Array($obj) || is_Object($obj)) {
                while (list($k,$v)=each($obj)) {
                        for ($i=0;$i<$indent;$i++) {
                                $return .= '&nbsp;';
                        }
                        $return .= $k.'->'.$v.'\n';
                        $return .= debugObject($v,$indent+1);
                }
                return $return;
        } 
}


/*
    /* Can't call this as @imap_setacl() as suppressing errors leads to 
       imap_errors() returning nothing. * /
    $result = imap_setacl($connection, $folder, $share_user, $acl_str);

    if (!$result) {
        $errors = imap_errors();
        if (is_array($errors)) {
            foreach ($errors as $err) {
                if ($err == 'ACL not available on this IMAP server') {
                    $err = _("This IMAP server does not support sharing folders.");
                    $result = 'no_support';
                }
                print_errormsg($err);
            }
        }
    } elseif (strlen($acl_str) < 1) {
        $errormsg = sprintf(_("All rights on folder '%s' successfully removed for user '%s'"), $folder, $share_user);
    } else {
        $errormsg = sprintf(_("User '%s' successfully given the following rights for the folder '%s': %s"), $share_user, $folder, $acl_str);
*/

////////////////////////////////////////////////////////////////
// ACL Functions
//
function getACL($connection, $folder) {
    $retval = sqimap_run_command ($connection, "GETACL \"$folder\"", 0, $response, $message);
    $rawrep = $retval[0];
    unset($retval);

    return parseACLreply($rawrep);
}

function delACL($connection, $folder, $user) {
    $retval = sqimap_run_command ($connection, "SETACL \"$folder\" $user \"\"", 0, $response, $message);
    // FIXME: Implement error handling
    return;
}

function setACL($connection, $folder, $user, $permstring) {
    // FIXME: Check for sane permstring.
    $retval = sqimap_run_command ($connection, "SETACL \"$folder\" $user $permstring", 0, $response, $message);
    // FIXME: Implement error handling
    return;
}

// code extracted from IMP and modified.
function parseACLreply($raw) {
    if (preg_match('/^\*\s+ACL\s+(.*)/i', $raw, $matches)) 
    {
        $res_arr = _atomise($matches[1]);
        $res_folder = array_shift($res_arr);
        $is_key = 1;
        $key = null;
        foreach ($res_arr as $var) {
            if ($is_key) {
                $key = $var;
                $is_key = 0;
            } else {
                $perms = preg_split('//', $var, -1, PREG_SPLIT_NO_EMPTY);
                foreach ($perms as $p_key => $p_var) {
                    $returnACL[$key][$p_var] = 1;
                }
                $is_key = 1;
            }
        }
        return $returnACL;
    }
    return false;
}

// Returns all subdirs _AND_ the basedir itself as an array.
function findSubdirs($connection, $basedir = "INBOX") {
    global $delimiter;
    sqgetGlobalVar('delimiter', $delimiter, SQ_SESSION);
    if (empty($delimiter))
        $delimiter = sqimap_get_delimiter($connection);
    $boxes = sqimap_mailbox_list($connection);
    foreach ($boxes as $boxes_part) {
        $box = $boxes_part['unformatted'];
        $lowerbox = strtolower($box);

        if ($basedir == $box || ereg("^".$basedir."\\$delimiter", $box)) {
            $retarray[] = $box;
        }
    }
    return $retarray;
}

////////////////////////////////////////////////////////////////
// Internationalisation
//
function IA_init_language() {
    global $ia_prevdomain, $ia_prevdomainpath;
    if (textdomain(NULL) != "imapacl") {
        $ia_prevdomain = textdomain(NULL);
        $ia_prevdomainpath = bindtextdomain('imapacl', SM_PATH . 'plugins/imapacl/locale');
        textdomain('imapacl');
    }
}

function IA_clean_language() {
    global $ia_prevdomain, $ia_prevdomainpath;
    if (textdomain(NULL) == "imapacl") {
    // Switch back to the SquirrelMail domain
        bindtextdomain($ia_prevdomain, $ia_prevdomainpath);
        textdomain($ia_prevdomain);
    }
}

function IA_set_SM_language() {
    bindtextdomain('squirrelmail', SM_PATH . 'locale');
    textdomain('squirrelmail');
}

////////////////////////////////////////////////////////////////
// Utility Functions
//
/**
 * Crudely split a string into 'atoms'
 * (code taken from IMP)
 *
 * @access private
 * @param string $in  The string to split.
 * @return array  An array of 'atoms'.
 */
function _atomise($in)
{
    $length = strlen($in);
    $qt = false;
    $idx = 0;
    $out = array();
    for ($i = 0; $i < $length; $i++) {
        $char = substr($in, $i, 1);
        if (($char == '"') && !$qt) {
            $qt = true;
            $idx++;
            continue;
        } elseif (($char == ' ') && !$qt) {
            $idx++;
        } elseif (($char == '"') && $qt) {
            $qt = false;
            $idx++;
        } else {
            if (empty($out[$idx])) {
                $out[$idx] = $char;
            } else {
                $out[$idx] .= $char;
            }
        }
    }
    return $out;
}

IA_clean_language();

?>
